imodels.tree.cart_ccp API documentation

# Module `imodels.tree.cart_ccp`

Expand source code

    from sklearn.tree import DecisionTreeClassifier, export_text, DecisionTreeRegressor
    from sklearn.base import BaseEstimator
    from sklearn import datasets
    from sklearn.datasets import make_friedman1,make_friedman2,make_friedman3
    from imodels.util.tree import compute_tree_complexity
    from copy import deepcopy

    class DecisionTreeClassifierCCP(DecisionTreeClassifier):
        def __init__(self, estimator_: BaseEstimator, desired_complexity: int = 1):
            self.desired_complexity = desired_complexity
            #print('est', estimator_)
            self.estimator_ = estimator_

        def fit(self,X,y,sample_weight=None,*args, **kwargs):
            path = self.estimator_.cost_complexity_pruning_path(X,y)
            ccp_alphas, impurities = path.ccp_alphas, path.impurities
            complexities = {}
            for alpha in ccp_alphas:
                est_params = self.estimator_.get_params()
                est_params['ccp_alpha'] = alpha
                copied_estimator =  deepcopy(self.estimator_).set_params(**est_params)
                copied_estimator.fit(X, y)
                complexities[alpha] = self._get_complexity(copied_estimator)
            closest_alpha, closest_leaves = min(complexities.items(), key=lambda x: abs(self.desired_complexity - x[1]))
            params_for_fitting = self.estimator_.get_params()
            params_for_fitting['ccp_alpha'] = closest_alpha
            self.estimator_.set_params(**params_for_fitting)
            self.estimator_.fit(X,y,*args, **kwargs)

        def _get_complexity(self,BaseEstimator):
            return compute_tree_complexity(BaseEstimator.tree_)

        def predict_proba(self, *args, **kwargs):
            if hasattr(self.estimator_, 'predict_proba'):
                return self.estimator_.predict_proba(*args, **kwargs)
            else:
                return NotImplemented

        def predict(self,X,*args, **kwargs):
            return self.estimator_.predict(X,*args, **kwargs)

        def score(self, *args, **kwargs):
            if hasattr(self.estimator_, 'score'):
                return self.estimator_.score(*args, **kwargs)
            else:
                return NotImplemented

    class DecisionTreeRegressorCCP(BaseEstimator):

        def __init__(self, estimator_: BaseEstimator, desired_complexity: int = 1):
            self.desired_complexity = desired_complexity
            #print('est', estimator_)
            self.estimator_ = estimator_

        def fit(self,X,y,sample_weight=None,*args, **kwargs):
            path = self.estimator_.cost_complexity_pruning_path(X,y)
            ccp_alphas, impurities = path.ccp_alphas, path.impurities
            complexities = {}
            for alpha in ccp_alphas:
                est_params = self.estimator_.get_params()
                est_params['ccp_alpha'] = alpha
                copied_estimator =  deepcopy(self.estimator_).set_params(**est_params)
                copied_estimator.fit(X, y)
                complexities[alpha] = self._get_complexity(copied_estimator)
            closest_alpha, closest_leaves = min(complexities.items(), key=lambda x: abs(self.desired_complexity - x[1]))
            params_for_fitting = self.estimator_.get_params()
            params_for_fitting['ccp_alpha'] = closest_alpha
            self.estimator_.set_params(**params_for_fitting)
            self.estimator_.fit(X,y,*args, **kwargs)

        def _get_complexity(self,BaseEstimator):
            return compute_tree_complexity(BaseEstimator.tree_)

        def predict(self,X,*args, **kwargs):
            return self.estimator_.predict(X,*args, **kwargs)

        def score(self, *args, **kwargs):
            if hasattr(self.estimator_, 'score'):
                return self.estimator_.score(*args, **kwargs)
            else:
                return NotImplemented

    if __name__ == '__main__':
        m = DecisionTreeClassifierCCP(estimator_=DecisionTreeClassifier(min_samples_leaf = 5),desired_complexity = 10)
        #X,y = make_friedman1()
        X, y = datasets.load_breast_cancer(return_X_y=True)
        m.fit(X,y)
        m.predict(X)
        m.score(X,y)

## Classes

`class DecisionTreeClassifierCCP (estimator_: sklearn.base.BaseEstimator, desired_complexity: int = 1)`

  A decision tree classifier.

  Read more in the :ref:`User Guide <tree>`.

  ## Parameters

  `criterion` :`{"gini", "entropy"}`, default=`"gini"`
    The function to measure the quality of a split. Supported criteria are "gini" for the Gini impurity and "entropy" for the information gain.
  `splitter` :`{"best", "random"}`, default=`"best"`
    The strategy used to choose the split at each node. Supported strategies are "best" to choose the best split and "random" to choose the best random split.
  `max_depth` :`int`, default=`None`
    The maximum depth of the tree. If None, then nodes are expanded until all leaves are pure or until all leaves contain less than min_samples_split samples.
  `min_samples_split` :`int` or `float`, default=`2`

    The minimum number of samples required to split an internal node:

    - If int, then consider `min_samples_split` as the minimum number.
    - If float, then `min_samples_split` is a fraction and `ceil(min_samples_split * n_samples)` are the minimum number of samples for each split.

    Changed in version: 0.18

    Added float values for fractions.

  `min_samples_leaf` :`int` or `float`, default=`1`

    The minimum number of samples required to be at a leaf node. A split point at any depth will only be considered if it leaves at least `min_samples_leaf` training samples in each of the left and right branches. This may have the effect of smoothing the model, especially in regression.

    - If int, then consider `min_samples_leaf` as the minimum number.
    - If float, then `min_samples_leaf` is a fraction and `ceil(min_samples_leaf * n_samples)` are the minimum number of samples for each node.

    Changed in version: 0.18

    Added float values for fractions.

  `min_weight_fraction_leaf` :`float`, default=`0.0`
    The minimum weighted fraction of the sum total of weights (of all the input samples) required to be at a leaf node. Samples have equal weight when sample_weight is not provided.
  `max_features` :`int, float` or `{"auto", "sqrt", "log2"}`, default=`None`

    The number of features to consider when looking for the best split:

        - If int, then consider <code>max\_features</code> features at each split.
        - If float, then <code>max\_features</code> is a fraction and
          `int(max_features * n_features)` features are considered at each
          split.
        - If "auto", then `max_features=sqrt(n_features)`.
        - If "sqrt", then `max_features=sqrt(n_features)`.
        - If "log2", then `max_features=log2(n_features)`.
        - If None, then `max_features=n_features`.

    Note: the search for a split does not stop until at least one valid partition of the node samples is found, even if it requires to effectively inspect more than `max_features` features.

  `random_state` :`int, RandomState instance` or `None`, default=`None`
    Controls the randomness of the estimator. The features are always randomly permuted at each split, even if `splitter` is set to `"best"`. When `max_features < n_features`, the algorithm will select `max_features` at random at each split before finding the best split among them. But the best found split may vary across different runs, even if `max_features=n_features`. That is the case, if the improvement of the criterion is identical for several splits and one split has to be selected at random. To obtain a deterministic behaviour during fitting, `random_state` has to be fixed to an integer. See :term:`Glossary <random_state>` for details.
  `max_leaf_nodes` :`int`, default=`None`
    Grow a tree with `max_leaf_nodes` in best-first fashion. Best nodes are defined as relative reduction in impurity. If None then unlimited number of leaf nodes.
  `min_impurity_decrease` :`float`, default=`0.0`

    A node will be split if this split induces a decrease of the impurity greater than or equal to this value.

    The weighted impurity decrease equation is the following::

        N_t / N * (impurity - N_t_R / N_t * right_impurity
                            - N_t_L / N_t * left_impurity)

    where `N` is the total number of samples, `N_t` is the number of samples at the current node, `N_t_L` is the number of samples in the left child, and `N_t_R` is the number of samples in the right child.

    `N`, `N_t`, `N_t_R` and `N_t_L` all refer to the weighted sum, if `sample_weight` is passed.

    Added in version: 0.19

  `class_weight` :`dict, list` of `dict` or `"balanced"`, default=`None`

    Weights associated with classes in the form `{class_label: weight}`. If None, all classes are supposed to have weight one. For multi-output problems, a list of dicts can be provided in the same order as the columns of y.

    Note that for multioutput (including multilabel) weights should be defined for each class of every column in its own dict. For example, for four-class multilabel classification weights should be [{0: 1, 1: 1}, {0: 1, 1: 5}, {0: 1, 1: 1}, {0: 1, 1: 1}] instead of [{1:1}, {2:5}, {3:1}, {4:1}].

    The "balanced" mode uses the values of y to automatically adjust weights inversely proportional to class frequencies in the input data as `n_samples / (n_classes * np.bincount(y))`

    For multi-output, the weights of each column of y will be multiplied.

    Note that these weights will be multiplied with sample_weight (passed through the fit method) if sample_weight is specified.

  `ccp_alpha` :`non-negative float`, default=`0.0`

    Complexity parameter used for Minimal Cost-Complexity Pruning. The subtree with the largest cost complexity that is smaller than `ccp_alpha` will be chosen. By default, no pruning is performed. See :ref:`minimal_cost_complexity_pruning` for details.

    Added in version: 0.22

  ## Attributes

  `classes_` :`ndarray` of `shape (n_classes,)` or `list` of `ndarray`
    The classes labels (single output problem), or a list of arrays of class labels (multi-output problem).
  `feature_importances_` :`ndarray` of `shape (n_features,)`

    The impurity-based feature importances. The higher, the more important the feature. The importance of a feature is computed as the (normalized) total reduction of the criterion brought by that feature. It is also known as the Gini importance [4]_.

    Warning: impurity-based feature importances can be misleading for high cardinality features (many unique values). See :func:`sklearn.inspection.permutation_importance` as an alternative.

  `max_features_` :`int`
    The inferred value of max_features.
  `n_classes_` :`int` or `list` of `int`
    The number of classes (for single output problems), or a list containing the number of classes for each output (for multi-output problems).
  `n_features_` :`int`

    The number of features when `fit` is performed.

    Deprecated since version: 1.0

    `n_features_` is deprecated in 1.0 and will be removed in 1.2. Use `n_features_in_` instead.

  `n_features_in_` :`int`

    Number of features seen during :term:`fit`.

    Added in version: 0.24

  `feature_names_in_` :`ndarray` of `shape (`n_features_in_`,)`

    Names of features seen during :term:`fit`. Defined only when `X` has feature names that are all strings.

    Added in version: 1.0

  `n_outputs_` :`int`
    The number of outputs when `fit` is performed.
  `tree_` :`Tree instance`
    The underlying Tree object. Please refer to `help(sklearn.tree._tree.Tree)` for attributes of Tree object and :ref:`sphx_glr_auto_examples_tree_plot_unveil_tree_structure.py` for basic usage of these attributes.

  ## See Also

  `DecisionTreeRegressor`
    A decision tree regressor.

  ## Notes

  The default values for the parameters controlling the size of the trees (e.g. `max_depth`, `min_samples_leaf`, etc.) lead to fully grown and unpruned trees which can potentially be very large on some data sets. To reduce memory consumption, the complexity and size of the trees should be controlled by setting those parameter values.

  The :meth:`predict` method operates using the :func:`numpy.argmax` function on the outputs of :meth:`predict_proba`. This means that in case the highest predicted probabilities are tied, the classifier will predict the tied class with the lowest index in :term:`classes_`.

  ## References

  .. [1] https://en.wikipedia.org/wiki/Decision_tree_learning

  .. [2] L. Breiman, J. Friedman, R. Olshen, and C. Stone, "Classification and Regression Trees", Wadsworth, Belmont, CA, 1984.

  .. [3] T. Hastie, R. Tibshirani and J. Friedman. "Elements of Statistical Learning", Springer, 2009.

  .. [4] L. Breiman, and A. Cutler, "Random Forests", https://www.stat.berkeley.edu/~breiman/RandomForests/cc_home.htm

  ## Examples

      >>> from sklearn.datasets import load_iris
      >>> from sklearn.model_selection import cross_val_score
      >>> from sklearn.tree import DecisionTreeClassifier
      >>> clf = DecisionTreeClassifier(random_state=0)
      >>> iris = load_iris()
      >>> cross_val_score(clf, iris.data, iris.target, cv=10)
      ...                             # doctest: +SKIP
      ...
      array([ 1.     ,  0.93...,  0.86...,  0.93...,  0.93...,
              0.93...,  0.93...,  1.     ,  0.93...,  1.      ])

  Expand source code

      class DecisionTreeClassifierCCP(DecisionTreeClassifier):
          def __init__(self, estimator_: BaseEstimator, desired_complexity: int = 1):
              self.desired_complexity = desired_complexity
              #print('est', estimator_)
              self.estimator_ = estimator_

          def fit(self,X,y,sample_weight=None,*args, **kwargs):
              path = self.estimator_.cost_complexity_pruning_path(X,y)
              ccp_alphas, impurities = path.ccp_alphas, path.impurities
              complexities = {}
              for alpha in ccp_alphas:
                  est_params = self.estimator_.get_params()
                  est_params['ccp_alpha'] = alpha
                  copied_estimator =  deepcopy(self.estimator_).set_params(**est_params)
                  copied_estimator.fit(X, y)
                  complexities[alpha] = self._get_complexity(copied_estimator)
              closest_alpha, closest_leaves = min(complexities.items(), key=lambda x: abs(self.desired_complexity - x[1]))
              params_for_fitting = self.estimator_.get_params()
              params_for_fitting['ccp_alpha'] = closest_alpha
              self.estimator_.set_params(**params_for_fitting)
              self.estimator_.fit(X,y,*args, **kwargs)

          def _get_complexity(self,BaseEstimator):
              return compute_tree_complexity(BaseEstimator.tree_)

          def predict_proba(self, *args, **kwargs):
              if hasattr(self.estimator_, 'predict_proba'):
                  return self.estimator_.predict_proba(*args, **kwargs)
              else:
                  return NotImplemented

          def predict(self,X,*args, **kwargs):
              return self.estimator_.predict(X,*args, **kwargs)

          def score(self, *args, **kwargs):
              if hasattr(self.estimator_, 'score'):
                  return self.estimator_.score(*args, **kwargs)
              else:
                  return NotImplemented

  ### Ancestors

  - sklearn.tree._classes.DecisionTreeClassifier
  - sklearn.base.ClassifierMixin
  - sklearn.tree._classes.BaseDecisionTree
  - sklearn.base.MultiOutputMixin
  - sklearn.base.BaseEstimator

  ### Methods

  `def fit(self, X, y, sample_weight=None, *args, **kwargs)`

    Build a decision tree classifier from the training set (X, y).

    ## Parameters

    `X` :`{array-like, sparse matrix}` of `shape (n_samples, n_features)`
      The training input samples. Internally, it will be converted to `dtype=np.float32` and if a sparse matrix is provided to a sparse `csc_matrix`.
    `y` :`array-like` of `shape (n_samples,)` or `(n_samples, n_outputs)`
      The target values (class labels) as integers or strings.
    `sample_weight` :`array-like` of `shape (n_samples,)`, default=`None`
      Sample weights. If None, then samples are equally weighted. Splits that would create child nodes with net zero or negative weight are ignored while searching for a split in each node. Splits are also ignored if they would result in any single class carrying a negative weight in either child node.
    `check_input` :`bool`, default=`True`
      Allow to bypass several input checking. Don't use this parameter unless you know what you do.
    `X_idx_sorted` :`deprecated`, default=`"deprecated"`

      This parameter is deprecated and has no effect. It will be removed in 1.1 (renaming of 0.26).

      Deprecated since version: 0.24

    ## Returns

    `self` :`DecisionTreeClassifier`
      Fitted estimator.
    Expand source code

        def fit(self,X,y,sample_weight=None,*args, **kwargs):
            path = self.estimator_.cost_complexity_pruning_path(X,y)
            ccp_alphas, impurities = path.ccp_alphas, path.impurities
            complexities = {}
            for alpha in ccp_alphas:
                est_params = self.estimator_.get_params()
                est_params['ccp_alpha'] = alpha
                copied_estimator =  deepcopy(self.estimator_).set_params(**est_params)
                copied_estimator.fit(X, y)
                complexities[alpha] = self._get_complexity(copied_estimator)
            closest_alpha, closest_leaves = min(complexities.items(), key=lambda x: abs(self.desired_complexity - x[1]))
            params_for_fitting = self.estimator_.get_params()
            params_for_fitting['ccp_alpha'] = closest_alpha
            self.estimator_.set_params(**params_for_fitting)
            self.estimator_.fit(X,y,*args, **kwargs)

  `def predict(self, X, *args, **kwargs)`

    Predict class or regression value for X.

    For a classification model, the predicted class for each sample in X is returned. For a regression model, the predicted value based on X is returned.

    ## Parameters

    `X` :`{array-like, sparse matrix}` of `shape (n_samples, n_features)`
      The input samples. Internally, it will be converted to `dtype=np.float32` and if a sparse matrix is provided to a sparse `csr_matrix`.
    `check_input` :`bool`, default=`True`
      Allow to bypass several input checking. Don't use this parameter unless you know what you do.

    ## Returns

    `y` :`array-like` of `shape (n_samples,)` or `(n_samples, n_outputs)`
      The predicted classes, or the predict values.
    Expand source code

        def predict(self,X,*args, **kwargs):
            return self.estimator_.predict(X,*args, **kwargs)

  `def predict_proba(self, *args, **kwargs)`

    Predict class probabilities of the input samples X.

    The predicted class probability is the fraction of samples of the same class in a leaf.

    ## Parameters

    `X` :`{array-like, sparse matrix}` of `shape (n_samples, n_features)`
      The input samples. Internally, it will be converted to `dtype=np.float32` and if a sparse matrix is provided to a sparse `csr_matrix`.
    `check_input` :`bool`, default=`True`
      Allow to bypass several input checking. Don't use this parameter unless you know what you do.

    ## Returns

    `proba` :`ndarray` of `shape (n_samples, n_classes)` or `list` of `n_outputs such arrays if n_outputs > 1`
      The class probabilities of the input samples. The order of the classes corresponds to that in the attribute :term:`classes_`.
    Expand source code

        def predict_proba(self, *args, **kwargs):
            if hasattr(self.estimator_, 'predict_proba'):
                return self.estimator_.predict_proba(*args, **kwargs)
            else:
                return NotImplemented

  `def score(self, *args, **kwargs)`

    Return the mean accuracy on the given test data and labels.

    In multi-label classification, this is the subset accuracy which is a harsh metric since you require for each sample that each label set be correctly predicted.

    ## Parameters

    `X` :`array-like` of `shape (n_samples, n_features)`
      Test samples.
    `y` :`array-like` of `shape (n_samples,)` or `(n_samples, n_outputs)`
      True labels for `X`.
    `sample_weight` :`array-like` of `shape (n_samples,)`, default=`None`
      Sample weights.

    ## Returns

    `score` :`float`
      Mean accuracy of `self.predict(X)` wrt. `y`.
    Expand source code

        def score(self, *args, **kwargs):
            if hasattr(self.estimator_, 'score'):
                return self.estimator_.score(*args, **kwargs)
            else:
                return NotImplemented

`class DecisionTreeRegressorCCP (estimator_: sklearn.base.BaseEstimator, desired_complexity: int = 1)`

  Base class for all estimators in scikit-learn.

  ## Notes

  All estimators should specify all the parameters that can be set at the class level in their `__init__` as explicit keyword arguments (no `*args` or `**kwargs`).

  Expand source code

      class DecisionTreeRegressorCCP(BaseEstimator):

          def __init__(self, estimator_: BaseEstimator, desired_complexity: int = 1):
              self.desired_complexity = desired_complexity
              #print('est', estimator_)
              self.estimator_ = estimator_

          def fit(self,X,y,sample_weight=None,*args, **kwargs):
              path = self.estimator_.cost_complexity_pruning_path(X,y)
              ccp_alphas, impurities = path.ccp_alphas, path.impurities
              complexities = {}
              for alpha in ccp_alphas:
                  est_params = self.estimator_.get_params()
                  est_params['ccp_alpha'] = alpha
                  copied_estimator =  deepcopy(self.estimator_).set_params(**est_params)
                  copied_estimator.fit(X, y)
                  complexities[alpha] = self._get_complexity(copied_estimator)
              closest_alpha, closest_leaves = min(complexities.items(), key=lambda x: abs(self.desired_complexity - x[1]))
              params_for_fitting = self.estimator_.get_params()
              params_for_fitting['ccp_alpha'] = closest_alpha
              self.estimator_.set_params(**params_for_fitting)
              self.estimator_.fit(X,y,*args, **kwargs)

          def _get_complexity(self,BaseEstimator):
              return compute_tree_complexity(BaseEstimator.tree_)

          def predict(self,X,*args, **kwargs):
              return self.estimator_.predict(X,*args, **kwargs)

          def score(self, *args, **kwargs):
              if hasattr(self.estimator_, 'score'):
                  return self.estimator_.score(*args, **kwargs)
              else:
                  return NotImplemented

  ### Ancestors

  - sklearn.base.BaseEstimator

  ### Methods

  `def fit(self, X, y, sample_weight=None, *args, **kwargs)`
    Expand source code

        def fit(self,X,y,sample_weight=None,*args, **kwargs):
            path = self.estimator_.cost_complexity_pruning_path(X,y)
            ccp_alphas, impurities = path.ccp_alphas, path.impurities
            complexities = {}
            for alpha in ccp_alphas:
                est_params = self.estimator_.get_params()
                est_params['ccp_alpha'] = alpha
                copied_estimator =  deepcopy(self.estimator_).set_params(**est_params)
                copied_estimator.fit(X, y)
                complexities[alpha] = self._get_complexity(copied_estimator)
            closest_alpha, closest_leaves = min(complexities.items(), key=lambda x: abs(self.desired_complexity - x[1]))
            params_for_fitting = self.estimator_.get_params()
            params_for_fitting['ccp_alpha'] = closest_alpha
            self.estimator_.set_params(**params_for_fitting)
            self.estimator_.fit(X,y,*args, **kwargs)

  `def predict(self, X, *args, **kwargs)`
    Expand source code

        def predict(self,X,*args, **kwargs):
            return self.estimator_.predict(X,*args, **kwargs)

  `def score(self, *args, **kwargs)`
    Expand source code

        def score(self, *args, **kwargs):
            if hasattr(self.estimator_, 'score'):
                return self.estimator_.score(*args, **kwargs)
            else:
                return NotImplemented